public inbox for linux-kernel@vger.kernel.org
 help / color / mirror / Atom feed
* [PATCH] disentangle printk's whilst oopsing on SMP
@ 2003-06-17 14:36 David Howells
  0 siblings, 0 replies; only message in thread
From: David Howells @ 2003-06-17 14:36 UTC (permalink / raw)
  To: Linus Torvalds; +Cc: Kernel Mailing List


Hi Linus,

The attached patch stops printk's issued on different processors from becoming
char-by-char interleaved whilst one of them is oopsing.

David

diff -uNr linux-2.5.72/kernel/printk.c linux-2.5.72-auto/kernel/printk.c
--- linux-2.5.72/kernel/printk.c	2003-06-17 15:01:58.000000000 +0100
+++ linux-2.5.72-auto/kernel/printk.c	2003-06-17 15:06:31.000000000 +0100
@@ -373,6 +373,9 @@
 		logged_chars++;
 }
 
+/* cpu currently holding logbuf_lock */
+static volatile int printk_cpu = -1;
+
 /*
  * This is printk.  It can be called from any context.  We want it to work.
  * 
@@ -395,8 +398,9 @@
 	static char printk_buf[1024];
 	static int log_level_unknown = 1;
 
-	if (oops_in_progress) {
-		/* If a crash is occurring, make sure we can't deadlock */
+	if (oops_in_progress && printk_cpu==smp_processor_id()) {
+		/* If a crash is occurring during printk() on this CPU,
+		 * make sure we can't deadlock */
 		spin_lock_init(&logbuf_lock);
 		/* And make sure that we print immediately */
 		init_MUTEX(&console_sem);
@@ -404,6 +408,7 @@
 
 	/* This stops the holder of console_sem just where we want him */
 	spin_lock_irqsave(&logbuf_lock, flags);
+	printk_cpu = smp_processor_id();
 
 	/* Emit the output into the temporary buffer */
 	va_start(args, fmt);

^ permalink raw reply	[flat|nested] only message in thread

only message in thread, other threads:[~2003-06-17 14:23 UTC | newest]

Thread overview: (only message) (download: mbox.gz follow: Atom feed
-- links below jump to the message on this page --
2003-06-17 14:36 [PATCH] disentangle printk's whilst oopsing on SMP David Howells

This is a public inbox, see mirroring instructions
for how to clone and mirror all data and code used for this inbox